<?php
namespace Teknisa\FIS\Controllers;

use Teknisa\FIS\Services\Fis46000ParametrizacaoCodigosApuracaoService;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DTO\Response\Message;
use Zeedhi\Framework\DTO\Request\Row;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DataSource\ParameterBag;


class Fis46000ParametrizacaoCodigosApuracaoController extends \Teknisa\FIS\Controllers\TeknisaCrud{

    protected $dataSourceName;
    protected $parameterBag;
    protected $crudService;
    protected $dataSourceNameToSave  = false;
    protected $service;

    public function __construct($crudService, ParameterBag $parameterBag, Fis46000ParametrizacaoCodigosApuracaoService $service) {
        $this->service = $service;
        parent::__construct($crudService, $parameterBag);
    }

    
	/*******************************************************************
	*                  Funções Referentes a aba                        *
	*                  Código da Receita                               *
	*******************************************************************/
    
    public function updateCodigoDaReceita(DTO\Request $request, DTO\Response $response) {
        try{
            $rows = $request->getDataSet()->getRows();
            if(!$this->service->isKeepingOldCodigoDaReceita($rows)){
                $this->service->validarCodigoDaReceita($rows[1]);
                $this->service->updateCodigoDaReceita($rows);
            }
        }
        catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 500));
        }

	}
	
	public function salvarCodigoReceitaEstadual(DTO\Request $request, DTO\Response $response) {
        try {
            $rows = $this->crudService->getRowsToSave($request, 'codreceitaest');
            $this->service->salvarCodigoReceitaEstadual($rows[0]);
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 500));
        }
    }
	
	
	
	/********************************************************************
	*                  Funções Referentes a aba                         *
	*                  Código de Ajuste                                 *
	********************************************************************/
    
    public function saveCodigoAjuste(DTO\Request $request, DTO\Response $response) {
        try {
            $rows = $this->crudService->getRowsToSave($request, 'codajusteefdicms');
            $this->service->saveCodigoAjuste($rows[0]);
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 500));
        }
    }

    public function salvarImportacaoTabelas(DTO\Request $request, DTO\Response $response) {
        try {
            $row = $request->getRow();
            $this->service->salvarImportacaoTabelas($row);
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 500));
        }
    }
	
}
