<?php
namespace Teknisa\FIS\Services;

use Teknisa\Libs\Util\Environment;
use Teknisa\Libs\Util\Utilities;
use Doctrine\ORM\EntityManager;
use Zeedhi\Framework\DTO\Row;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DataSource\DataSet;
use Teknisa\Libs\Util\NewCode;
use Zeedhi\Framework\DataSource\FilterCriteria;
use DateTime;

class Fis09200ParametrosEmissaoGuiasService {

    protected $nrorg;
    protected $environment;
    protected $entityManager;
    protected $dataSourceManager;
    protected $newCode;
    protected $crudService;
    protected $fileUploader;
    protected $_fileServerUrl;
    
    public function __construct(Environment $environment, EntityManager $entityManager, $dataSourceManager, NewCode $newCode, $crudService, $fileUploader, $fileServerUrl) {
        $this->environment       = $environment;
        $this->entityManager     = $entityManager;
        $this->dataSourceManager = $dataSourceManager;
        $this->newCode           = $newCode;
        $this->nrorg             = $this->environment->getNrOrg();
        $this->crudService       = $crudService;
        $this->fileUploader      = $fileUploader;
        $this->_fileServerUrl    = $fileServerUrl;
    }

    public function geraNovoCodigo($contador, $nrorg){
        try {
            return $this->newCode->getCodeByTableName($contador, $nrorg);
        }catch (\Exception $e) {
            throw $e;
        }
    }

    public function salvarBeneficiario($row){
        try {
            $connection = $this->entityManager->getConnection();
            $connection->beginTransaction();
            if($row['__is_new']){
                $this->validaBeneficiarioExistente($row);
                $row['NRSEQPARAMBENIMP'] =  $this->geraNovoCodigo('PARAMBENEFIMP',$this->nrorg);
            }
            $dataSet = new DataSet('parambenefimp', [$row]);
            $this->dataSourceManager->persist($dataSet);
            $this->entityManager->flush();
            $connection->commit();
        }catch (\Exception $e) {
            throw $e;
        }
    }

    public function validaBeneficiarioExistente ($row){
        switch($row['IDTPPARAMBENEF']) {
            case 'F':
                $conditions     = $this->condicaoFederal($row);
                $resultadoBanco = $this->buscar($conditions,'parambenefimp');
                if( ! empty ($resultadoBanco->getRows() ) ){
                    throw new \Exception("País já cadastrado.", 1);
                }
                break;

            case 'E':
                $conditions     = $this->condicaoEstadual($row);
                $resultadoBanco = $this->buscar($conditions,'parambenefimp');
                if( ! empty ($resultadoBanco->getRows() ) ){
                    throw new \Exception("Estado já cadastrado.", 1);
                }
                break;

            case 'M':
                $conditions     = $this->condicaoMunicipal($row);
                $resultadoBanco = $this->buscar($conditions,'parambenefimp');
                if( ! empty ($resultadoBanco->getRows() ) ){
                    throw new \Exception("Município já cadastrado.", 1);
                }
                break;
        }
    }

    public function condicaoFederal($row) {
        $conditions = array(
            array(
                "value" => $this->nrorg,
                "operator" => "=",
                "columnName" => "NRORG"
            ),
            array(
                "value" => $row['CDPAIS'],
                "operator" => "=",
                "columnName" => "CDPAIS"
            ),
            array(
                "value" => 'F',
                "operator" => "=",
                "columnName" => "IDTPPARAMBENEF"
            )

        );
        return $conditions;
    }

    public function condicaoEstadual($row) {
        $conditions = array(
            array(
                "value" => $this->nrorg,
                "operator" => "=",
                "columnName" => "NRORG"
            ),
            array(
                "value" => $row['CDPAIS'],
                "operator" => "=",
                "columnName" => "CDPAIS"
            ),
            array(
                "value" => $row['SGESTADO'],
                "operator" => "=",
                "columnName" => "SGESTADO"
            ),
            array(
                "value" => 'E',
                "operator" => "=",
                "columnName" => "IDTPPARAMBENEF"
            )

        );
        return $conditions;
    }

    public function condicaoMunicipal($row) {
        $conditions = array(
            array(
                "value" => $this->nrorg,
                "operator" => "=",
                "columnName" => "NRORG"
            ),
            array(
                "value" => $row['CDPAIS'],
                "operator" => "=",
                "columnName" => "CDPAIS"
            ),
            array(
                "value" => $row['SGESTADO'],
                "operator" => "=",
                "columnName" => "SGESTADO"
            ),
            array(
                "value" => $row['CDMUNICIPIO'],
                "operator" => "=",
                "columnName" => "CDMUNICIPIO"
            ),
            array(
                "value" => 'M',
                "operator" => "=",
                "columnName" => "IDTPPARAMBENEF"
            )

        );
        return $conditions;
    }

    public function buscar($conditions, $dataSourceName){
        $filterCriteria = new FilterCriteria($dataSourceName,$conditions);
        return  $this->dataSourceManager->findBy($filterCriteria);
    }
    
    /* Certificado */
    
    public function salvarCertificado($row) {
        if($row['__is_new']) { //Apenas para novas incluisões de certificados
            $row['NMCERTIFICADO']      = $row['UPLOADCERT'][0]['name'];
            //$row['DSSENHACERTIFICADO'] = $this->encrypt($row['DSSENHACERTIFICADO']);
            $row['DSSENHACERTIFICADO'] = Utilities::zeedhiEncrypt($row['DSSENHACERTIFICADO']);
        } else {
            if($row['DSSENHACERTIFICADO'] != $row['__backupRow']['DSSENHACERTIFICADO']) {
                $row['DSSENHACERTIFICADO'] = Utilities::zeedhiEncrypt($row['DSSENHACERTIFICADO']);
            }
        }
        $row['DSPATHCERTIFICADO'] = $this->saveCertificadoInFileUplouder($row['UPLOADCERT'][0]);
        
        unset($row['__backupRow']);
        
        $connection = $this->entityManager->getConnection();
        $connection->beginTransaction();
        $dataSet = new DataSet('paramtrans', [$row]);
        $this->dataSourceManager->persist($dataSet);
        $this->entityManager->flush();
        $connection->commit();
    }
    
    public function saveCertificadoInFileUplouder($arquivo) {
        
        if (isset($arquivo)) {
            
            $file = array(
                "name" => time().'.pfx', 
                "lastModified" => $arquivo["lastModified"],
                "lastModifiedDate" => $arquivo["lastModifiedDate"],
                "webkitRelativePath" => $arquivo["webkitRelativePath"],
                "type" => $arquivo["type"],
                "size"  => $arquivo["size"], 
                "b64File" => $arquivo["b64File"]);
                
            $row['CERT']    = array();
            $row['CERT'][0] = $file;
            
            $this->fileUploader->uploadFilesInRow($row, 'CERT', 'Certificados');
            
            return $row['CERT'][0]['path'];
            
            /*
            
            $row = [
                'FILE' => [
                    [
                        "name" => time().'.'.explode('/', $arquivo["type"])[1], 
                        "type" => $arquivo["type"], 
                        "size" => $arquivo["size"], 
                        "path" => '', 
                        "b64File" => $arquivo["b64File"]
                    ]
                ],
                'FOLDER' => 'ANEXOLANCBOLPAG'
            ];
            
            return $this->fileUploader->uploadFile($row)[0]->getRows()[0]['FILE'][0]['PATH'];
            */
            
        } else {
            return false;
        }
    }

    private function encrypt($text, $salt = 'teknisa') {
        return trim(
            base64_encode(
                mcrypt_encrypt(
                    MCRYPT_RIJNDAEL_256,
                    str_pad($salt, 32, "\0"),
                    $text,
                    MCRYPT_MODE_ECB,
                    mcrypt_create_iv(mcrypt_get_iv_size(MCRYPT_RIJNDAEL_256, MCRYPT_MODE_ECB), MCRYPT_RAND)
                )
            )
        );
    }
    
    public function decrypt($text, $salt = 'teknisa'){
        return trim(
            mcrypt_decrypt(
                MCRYPT_RIJNDAEL_256,
                str_pad($salt, 32, "\0"),
                base64_decode($text),
                MCRYPT_MODE_ECB,
                mcrypt_create_iv(mcrypt_get_iv_size(MCRYPT_RIJNDAEL_256, MCRYPT_MODE_ECB), MCRYPT_RAND)
            )
        );
    }
}